import { downloadEscalationApi } from '@/services/task-api/download';
import { MaterialSourceEnum } from '@/utils/constEnum';
import { useRequest, useSize } from 'ahooks';
import { Button, Card, Divider, Flex, Skeleton, Space, Spin, Tag, Typography } from 'antd';
import React, { useEffect, useMemo, useRef, useState } from 'react';
import InfiniteScroll from 'react-infinite-scroll-component';
import VideoOpus from './components/VideoOpus';
import style from './index.less';

interface Props {
  data: any[];
  total: number;
  loading?: boolean;
  loadMore?: () => void;
  flexNum?: number;
  id?: string;
}

const OpusRoll: React.FC<Props> = ({
  loading,
  data,
  total,
  loadMore,
  flexNum = 0,
  id = 'scrollableDiv',
}) => {
  /************************/
  const ref = useRef<HTMLDivElement>(null);
  const size = useSize(ref);
  const [rowNum, setRowNum] = useState<number>(flexNum);
  const [dataList, setDataList] = useState<any[]>([]);
  const downloadEscalation = useRequest(downloadEscalationApi, { manual: true });
  /************************/

  useEffect(() => {
    setDataList(data || []);
  }, [data]);

  // 根据内容宽度计算列数
  useEffect(() => {
    if (size?.width && !flexNum) {
      let rowNum = Math.floor((size?.width - 26) / 260);
      setRowNum(rowNum || 1);
    }
  }, [size?.width]);

  const download = (url: string, userMaterialId: number) => {
    downloadEscalation.runAsync({ userMaterialId }).then(() => {
      const fileName = 'downloaded.mp4'; // 可以自定义下载的文件名
      let link = url;
      let x = new XMLHttpRequest();
      x.open('GET', link, true);
      x.responseType = 'blob';
      x.onload = () => {
        let url = window.URL.createObjectURL(x.response);
        let a = document.createElement('a');
        a.href = url;
        a.download = fileName;
        a.click();
      };
      x.send();
    });
  };

  const content = useMemo(() => {
    let sourceData: any[] = [];
    dataList.forEach((item, index) => {
      sourceData[index % rowNum]
        ? sourceData[index % rowNum]?.push(item)
        : (sourceData[index % rowNum] = [item]);
    });

    // 不足
    if (sourceData.length < rowNum) {
      let fewNum = rowNum - sourceData.length;
      let newData = Array(fewNum).fill([]);
      sourceData = [...sourceData, ...newData];
    }
    return (
      <>
        {sourceData.map((item: any[], index) => (
          <div
            key={index}
            className={style.content_row}
            style={{ width: (1 / rowNum) * 100 + '%' }}
          >
            {item?.map((list: any) => {
              const { size } = JSON.parse(list.materialClaimJson);
              return (
                <Card
                  className={style.left_content_card}
                  key={list.id}
                  bodyStyle={{ padding: 10, position: 'relative' }}
                  headStyle={{ padding: '5px 10px' }}
                  cover={
                    <div className={style.left_content_caover}>
                      <VideoOpus src={list.link} />
                    </div>
                  }
                  title={
                    <Space direction="vertical" size={0} style={{ width: '100%' }}>
                      <Typography.Title style={{ margin: 0, fontSize: 15 }} level={5} ellipsis>
                        {list.taskName}
                      </Typography.Title>
                      <Typography.Text type="secondary" style={{ fontSize: 13 }}>
                        {list.createTime}
                      </Typography.Text>
                    </Space>
                  }
                >
                  <Flex justify="space-between">
                    <Typography.Text>
                      素材尺寸：{size[0]}*{size[1]}
                    </Typography.Text>
                    <Typography.Text>
                      素材大小：{(list.size / 1024 / 1024).toFixed(2)}M
                    </Typography.Text>
                  </Flex>
                  <Flex justify="space-between">
                    <Typography.Text>素材ID：{list.taskId}</Typography.Text>
                    {!list.cost && list.cost !== 0 ? null : (
                      <Typography.Text>消耗：{list.cost}</Typography.Text>
                    )}
                  </Flex>
                  <Flex justify="space-between">
                    <Space size={0}>
                      <Tag color="#55acee">{(MaterialSourceEnum as any)[list.materialSource]}</Tag>
                      {list.checkStatus ? (
                        <Tag color="success">合格</Tag>
                      ) : (
                        <Tag color="warning">审核中</Tag>
                      )}
                    </Space>
                    <Space>
                      <Button
                        type="link"
                        style={{ padding: 0 }}
                        onClick={() => download(list.link, list.id)}
                      >
                        下载
                      </Button>
                      {/* <Button type="link" style={{ padding: 0 }}>详情</Button> */}
                    </Space>
                  </Flex>
                </Card>
              );
            })}
          </div>
        ))}
      </>
    );
  }, [rowNum, size, dataList]);

  const loadMoreData = () => {
    console.log('触发了--->');
    if (dataList.length === total) {
      return;
    }
    loadMore?.();
  };

  return (
    <div className={style.left_root}>
      <div className={style.left_content} ref={ref}>
        <InfiniteScroll
          dataLength={dataList.length}
          next={loadMoreData}
          hasMore={dataList.length < total}
          loader={
            <div style={{ display: 'flex', marginTop: 10 }}>
              {Array(rowNum)
                .fill('')
                .map((item, index) => (
                  <div
                    key={index}
                    className={style.content_row}
                    style={{
                      width: (1 / rowNum) * 100 + '%',
                      padding: '0 5px',
                      boxSizing: 'border-box',
                    }}
                  >
                    <Skeleton.Input active style={{ width: '100%' }} />
                  </div>
                ))}
            </div>
          }
          endMessage={<Divider plain>It is all, nothing more 🤐</Divider>}
          scrollableTarget={id}
          // height={height}
        >
          <div className={style.memo_content}>{content}</div>
        </InfiniteScroll>
      </div>
      {loading && (
        <div className={style.content_spin}>
          <Spin />
        </div>
      )}
    </div>
  );
};

export default React.memo(OpusRoll);
